@@ -60,12 +60,14 @@ class FrontendComm(CommBase):
             self._comm_name, self._comm_open)
 
         self.comm_port = None
+        self.register_call_handler('_send_comm_config',
+                                   self._send_comm_config)
 
         # self.kernel.parent is IPKernelApp unless we are in tests
         if self.kernel.parent:
             # Create a new socket
-            context = zmq.Context()
-            self.comm_socket = context.socket(zmq.ROUTER)
+            self.context = zmq.Context()
+            self.comm_socket = self.context.socket(zmq.ROUTER)
             self.comm_socket.linger = 1000
 
             self.comm_port = get_free_port()
@@ -88,13 +90,18 @@ class FrontendComm(CommBase):
 
                 def close():
                     """Close comm_socket_thread."""
-                    self.comm_thread_close.set()
-                    context.term()
-                    self.comm_socket_thread.join()
+                    self.close_thread()
                     parent_close()
 
                 self.kernel.parent.close = close
 
+    def close_thread(self):
+        """Close comm."""
+        self.comm_thread_close.set()
+        self.comm_socket.close()
+        self.context.term()
+        self.comm_socket_thread.join()
+
     def poll_thread(self):
         """Receive messages from comm socket."""
         if not PY2:
@@ -112,6 +119,8 @@ class FrontendComm(CommBase):
             out_stream = self.kernel.shell_streams[0]
         try:
             ident, msg = self.kernel.session.recv(self.comm_socket, 0)
+        except zmq.error.ContextTerminated:
+            return
         except Exception:
             self.kernel.log.warning("Invalid Message:", exc_info=True)
             return
@@ -147,24 +156,35 @@ class FrontendComm(CommBase):
             callback=callback,
             timeout=timeout)
 
-    # --- Private --------
-    def _wait_reply(self, call_id, call_name, timeout):
-        """Wait until the frontend replies to a request."""
-        if call_id in self._reply_inbox:
-            return
-
+    def wait_until(self, condition, timeout=None):
+        """Wait until condition is met. Returns False if timeout."""
+        if condition():
+            return True
         t_start = time.time()
-        while call_id not in self._reply_inbox:
-            if time.time() > t_start + timeout:
-                raise TimeoutError(
-                    "Timeout while waiting for '{}' reply".format(
-                        call_name))
+        while not condition():
+            if timeout is not None and time.time() > t_start + timeout:
+                return False
             if threading.current_thread() is self.comm_socket_thread:
                 # Wait for a reply on the comm channel.
                 self.poll_one()
             else:
                 # Wait 10ms for a reply
                 time.sleep(0.01)
+        return True
+
+    # --- Private --------
+    def _wait_reply(self, call_id, call_name, timeout, retry=True):
+        """Wait until the frontend replies to a request."""
+        def reply_received():
+            """The reply is there!"""
+            return call_id in self._reply_inbox
+        if not self.wait_until(reply_received):
+            if retry:
+                self._wait_reply(call_id, call_name, timeout, False)
+                return
+            raise TimeoutError(
+                "Timeout while waiting for '{}' reply.".format(
+                    call_name))
 
     def _comm_open(self, comm, msg):
         """
@@ -173,8 +193,17 @@ class FrontendComm(CommBase):
         self.calling_comm_id = comm.comm_id
         self._register_comm(comm)
         self._set_pickle_protocol(msg['content']['data']['pickle_protocol'])
-        self.remote_call()._set_pickle_protocol(pickle.HIGHEST_PROTOCOL)
+        self._send_comm_config()
+
+    def on_outgoing_call(self, call_dict):
+        """A message is about to be sent"""
+        call_dict["comm_port"] = self.comm_port
+        return super(FrontendComm, self).on_outgoing_call(call_dict)
+
+    def _send_comm_config(self):
+        """Send the comm config to the frontend."""
         self.remote_call()._set_comm_port(self.comm_port)
+        self.remote_call()._set_pickle_protocol(pickle.HIGHEST_PROTOCOL)
 
     def _comm_close(self, msg):
         """Close comm."""
@@ -200,3 +229,34 @@ class FrontendComm(CommBase):
                 comm._msg_callback(msg)
         comm.handle_msg = handle_msg
         super(FrontendComm, self)._register_comm(comm)
+
+    def _remote_callback(self, call_name, call_args, call_kwargs):
+        """Call the callback function for the remote call."""
+        saved_stdout_write = sys.stdout.write
+        saved_stderr_write = sys.stderr.write
+        sys.stdout.write = WriteWrapper(saved_stdout_write, call_name)
+        sys.stderr.write = WriteWrapper(saved_stderr_write, call_name)
+        try:
+            return super(FrontendComm, self)._remote_callback(
+                call_name, call_args, call_kwargs)
+        finally:
+            sys.stdout.write = saved_stdout_write
+            sys.stderr.write = saved_stderr_write
+
+
+class WriteWrapper():
+    """Wrapper to warn user when text is printed."""
+
+    def __init__(self, write, name):
+        self._write = write
+        self._name = name
+        self._warning_shown = False
+
+    def __call__(self, string):
+        """Print warning once."""
+        if not self._warning_shown:
+            self._warning_shown = True
+            self._write(
+                "\nOutput from spyder call "
+                + repr(self._name) + ":\n")
+        return self._write(string)
