@@ -13,6 +13,7 @@ Utilities
 from __future__ import print_function
 
 from itertools import islice
+import inspect
 import re
 
 # Local imports
@@ -35,7 +36,7 @@ class FakeObject(object):
 # Numpy arrays and numeric types support
 #==============================================================================
 try:
-    from numpy import (ndarray, array, matrix, recarray,
+    from numpy import (ndarray, array, matrix, recarray, integer,
                        int64, int32, int16, int8, uint64, uint32, uint16, uint8,
                        float64, float32, float16, complex64, complex128, bool_)
     from numpy.ma import MaskedArray
@@ -47,6 +48,11 @@ except:
      float64 = float32 = float16 = complex64 = complex128 = bool_ = FakeObject
 
 
+NUMERIC_NUMPY_TYPES = (int64, int32, int16, int8, uint64, uint32, uint16,
+                       uint8, float64, float32, float16, complex64, complex128,
+                       bool_)
+
+
 def get_numpy_dtype(obj):
     """Return NumPy data type associated to obj
     Return None if NumPy is not available
@@ -112,18 +118,33 @@ def try_to_eval(value):
     except (NameError, SyntaxError, ImportError):
         return value
 
-    
+
 def get_size(item):
-    """Return size of an item of arbitrary type"""
-    if isinstance(item, (list, set, tuple, dict)):
-        return len(item)
-    elif isinstance(item, (ndarray, MaskedArray)):
-        return item.shape
-    elif isinstance(item, Image):
-        return item.size
-    if isinstance(item, (DataFrame, Index, Series)):
-        return item.shape
-    else:
+    """Return shape/size/len of an item of arbitrary type"""
+    try:
+        if hasattr(item, 'shape') and isinstance(item.shape, (tuple, integer)):
+            try:
+                if item.shape:
+                    return item.shape
+                else:
+                    # Scalar value
+                    return 1
+            except RecursionError:
+                # This is necessary to avoid an error when trying to
+                # get the shape of these objects.
+                # Fixes spyder-ide/spyder-kernels#217
+                return (-1, -1)
+        elif hasattr(item, 'size') and isinstance(item.size, (tuple, integer)):
+            try:
+                return item.size
+            except RecursionError:
+                return (-1, -1)
+        elif hasattr(item, '__len__'):
+            return len(item)
+        else:
+            return 1
+    except Exception:
+        # There is one item
         return 1
 
 
@@ -240,16 +261,20 @@ def is_editable_type(value):
 #==============================================================================
 # Sorting
 #==============================================================================
-def sort_against(list1, list2, reverse=False):
+def sort_against(list1, list2, reverse=False, sort_key=None):
     """
     Arrange items of list1 in the same order as sorted(list2).
 
     In other words, apply to list1 the permutation which takes list2 
     to sorted(list2, reverse).
     """
+    if sort_key is None:
+        key = lambda x: x[0]
+    else:
+        key = lambda x: sort_key(x[0])
     try:
         return [item for _, item in 
-                sorted(zip(list2, list1), key=lambda x: x[0], reverse=reverse)]
+                sorted(zip(list2, list1), key=key, reverse=reverse)]
     except:
         return list1
 
@@ -268,11 +293,19 @@ def default_display(value, with_module=True):
     try:
         name = object_type.__name__
         module = object_type.__module__
+
+        # Classes correspond to new types
+        if name == 'type':
+            name = 'class'
+
         if with_module:
+            if name == 'module':
+                return value.__name__ + ' module'
+            if module == 'builtins':
+                return name + ' object'
             return name + ' object of ' + module + ' module'
-        else:
-            return name
-    except:
+        return name
+    except Exception:
         type_str = to_text_string(object_type)
         return type_str[1:-1]
 
@@ -331,10 +364,6 @@ def value_to_display(value, minmax=False, level=0):
     np_printoptions = FakeObject
 
     try:
-        numeric_numpy_types = (int64, int32, int16, int8,
-                               uint64, uint32, uint16, uint8,
-                               float64, float32, float16,
-                               complex128, complex64, bool_)
         if ndarray is not FakeObject:
             # Save printoptions
             np_printoptions = get_printoptions()
@@ -355,11 +384,11 @@ def value_to_display(value, minmax=False, level=0):
                     try:
                         display = 'Min: %r\nMax: %r' % (value.min(), value.max())
                     except (TypeError, ValueError):
-                        if value.dtype.type in numeric_numpy_types:
+                        if value.dtype.type in NUMERIC_NUMPY_TYPES:
                             display = str(value)
                         else:
                             display = default_display(value)
-                elif value.dtype.type in numeric_numpy_types:
+                elif value.dtype.type in NUMERIC_NUMPY_TYPES:
                     display = str(value)
                 else:
                     display = default_display(value)
@@ -432,7 +461,7 @@ def value_to_display(value, minmax=False, level=0):
             display = str(value)
         elif (isinstance(value, NUMERIC_TYPES) or
               isinstance(value, bool) or
-              isinstance(value, numeric_numpy_types)):
+              isinstance(value, NUMERIC_NUMPY_TYPES)):
             display = repr(value)
         else:
             if level == 0:
@@ -518,11 +547,16 @@ def get_type_string(item):
         return type(item).__name__
     if isinstance(item, Series):
         return "Series"
+
     found = re.findall(r"<(?:type|class) '(\S*)'>",
                        to_text_string(type(item)))
     if found:
+        if found[0] == 'type':
+            return 'class'
         return found[0]
-    
+    else:
+        return None
+
 
 def is_known_type(item):
     """Return True if object has a known type"""
@@ -533,13 +567,13 @@ def is_known_type(item):
 def get_human_readable_type(item):
     """Return human-readable type string of an item"""
     if isinstance(item, (ndarray, MaskedArray)):
-        return item.dtype.name
+        return u'Array of ' + item.dtype.name
     elif isinstance(item, Image):
         return "Image"
     else:
         text = get_type_string(item)
         if text is None:
-            text = to_text_string('unknown')
+            text = to_text_string('Unknown')
         else:
             return text[text.find('.')+1:]
 
@@ -549,11 +583,13 @@ def get_human_readable_type(item):
 # CollectionsEditor)
 #==============================================================================
 def is_supported(value, check_all=False, filters=None, iterate=False):
-    """Return True if the value is supported, False otherwise"""
+    """Return True if value is supported, False otherwise."""
     assert filters is not None
     if value is None:
         return True
-    if not is_editable_type(value):
+    if is_callable_or_module(value):
+        return True
+    elif not is_editable_type(value):
         return False
     elif not isinstance(value, filters):
         return False
@@ -577,21 +613,32 @@ def is_supported(value, check_all=False, filters=None, iterate=False):
     return True
 
 
+def is_callable_or_module(value):
+    """Return True if value is a callable or module, False otherwise."""
+    try:
+        callable_or_module = callable(value) or inspect.ismodule(value)
+    except Exception:
+        callable_or_module = False
+    return callable_or_module
+
+
 def globalsfilter(input_dict, check_all=False, filters=None,
                   exclude_private=None, exclude_capitalized=None,
                   exclude_uppercase=None, exclude_unsupported=None,
-                  excluded_names=None):
-    """Keep only objects that can be pickled"""
+                  excluded_names=None, exclude_callables_and_modules=None):
+    """Keep objects in namespace view according to different criteria."""
     output_dict = {}
     for key, value in list(input_dict.items()):
-        excluded = (exclude_private and key.startswith('_')) or \
-                   (exclude_capitalized and key[0].isupper()) or \
-                   (exclude_uppercase and key.isupper()
-                    and len(key) > 1 and not key[1:].isdigit()) or \
-                   (key in excluded_names) or \
-                   (exclude_unsupported and \
-                    not is_supported(value, check_all=check_all,
-                                     filters=filters))
+        excluded = (
+            (exclude_private and key.startswith('_')) or
+            (exclude_capitalized and key[0].isupper()) or
+            (exclude_uppercase and key.isupper() and
+             len(key) > 1 and not key[1:].isdigit()) or
+            (key in excluded_names) or
+            (exclude_callables_and_modules and is_callable_or_module(value)) or
+            (exclude_unsupported and
+             not is_supported(value, check_all=check_all, filters=filters))
+        )
         if not excluded:
             output_dict[key] = value
     return output_dict
@@ -603,7 +650,7 @@ def globalsfilter(input_dict, check_all=False, filters=None,
 REMOTE_SETTINGS = ('check_all', 'exclude_private', 'exclude_uppercase',
                    'exclude_capitalized', 'exclude_unsupported',
                    'excluded_names', 'minmax', 'show_callable_attributes',
-                   'show_special_attributes')
+                   'show_special_attributes', 'exclude_callables_and_modules')
 
 
 def get_supported_types():
@@ -624,7 +671,7 @@ def get_supported_types():
     except:
         pass
     try:
-        from pandas import DataFrame, Series, DatetimeIndex
+        from pandas import DataFrame, Series, Index
         editable_types += [DataFrame, Series, Index]
     except:
         pass
@@ -650,13 +697,16 @@ def get_remote_data(data, settings, mode, more_excluded_names=None):
     excluded_names = settings['excluded_names']
     if more_excluded_names is not None:
         excluded_names += more_excluded_names
-    return globalsfilter(data, check_all=settings['check_all'],
-                         filters=tuple(supported_types[mode]),
-                         exclude_private=settings['exclude_private'],
-                         exclude_uppercase=settings['exclude_uppercase'],
-                         exclude_capitalized=settings['exclude_capitalized'],
-                         exclude_unsupported=settings['exclude_unsupported'],
-                         excluded_names=excluded_names)
+    return globalsfilter(
+        data,
+        check_all=settings['check_all'],
+        filters=tuple(supported_types[mode]),
+        exclude_private=settings['exclude_private'],
+        exclude_uppercase=settings['exclude_uppercase'],
+        exclude_capitalized=settings['exclude_capitalized'],
+        exclude_unsupported=settings['exclude_unsupported'],
+        exclude_callables_and_modules=settings['exclude_callables_and_modules'],
+        excluded_names=excluded_names)
 
 
 def make_remote_view(data, settings, more_excluded_names=None):
@@ -673,4 +723,4 @@ def make_remote_view(data, settings, more_excluded_names=None):
                        'size':  get_size(value),
                        'color': get_color_name(value),
                        'view':  view}
-    return remote
+    return remote
\ No newline at end of file
